//---------------------------------------------------------------------------
// audio.fs    Audio module
//
// 2008 written by Phillip Trelford
//---------------------------------------------------------------------------

#light

module Audio

// Reference: http://msdn2.microsoft.com/en-us/library/bb203895.aspx

    open Microsoft.Xna.Framework.Audio

    let mutable engine = None
    let mutable waveBank = None
    let mutable soundBank = None
    
    let Initialize () =
        engine <- Some (new AudioEngine ("Content\\Audio\\LunarLanderAudio.xgs"))
        waveBank <- Some (new WaveBank(Option.get engine, "Content\\Audio\\Wave Bank.xwb"))
        soundBank <- Some (new SoundBank(Option.get engine, "Content\\Audio\\Sound Bank.xsb"))

    let Update () =
        (Option.get engine).Update ()
        
    let mutable thrustCue = None        
        
    let PlayThrust isThrusting =        
        if isThrusting then           
            match thrustCue with
            | None -> let soundBank = Option.get soundBank in let cue = soundBank.GetCue ("thrust") in thrustCue <- Some (cue); cue.Play () 
            | Some (cue) -> if cue.IsPaused then cue.Resume ()
        else
            match thrustCue with
            | None -> ()
            | Some (cue) -> if cue.IsPlaying then cue.Pause ()
                        
    let PlayExplosion () =
        let cue = let soundBank = Option.get soundBank in soundBank.GetCue ("explosion")
        cue.Play ()
           