//---------------------------------------------------------------------------
// geometry.fs    Measure definitions specific to this project
//
// 2008 written by Phillip Trelford
//---------------------------------------------------------------------------

#light

namespace MicrosoftResearch.LunarLander

#nowarn "065"       // For the "struct" feature (which is still not fully tested)

open Microsoft.Xna.Framework

type Line2 =
    struct
        val V1 : Vector2
        val V2 : Vector2
        new (v1,v2) = {V1 = v1; V2 = v2}
        
        member line.IsBounded (v:Vector2) =    
            let isBoundedX =            
                if line.V1.X > line.V2.X then
                    v.X <= line.V1.X && v.X >= line.V2.X
                else 
                    v.X >= line.V1.X && v.X <= line.V2.X
            let isBoundedY =
              if line.V1.Y > line.V2.Y then
                    v.Y <= line.V1.Y && v.Y >= line.V2.Y
                else
                    v.Y >= line.V1.Y && v.Y <= line.V2.Y                
            isBoundedX && isBoundedY                 
            
        member line.ComputeDX () = (line.V2.Y - line.V1.Y)/(line.V2.X - line.V1.X)
        
        member line.ComputeKY () = line.V1.Y - (line.V1.X*line.ComputeDX ())                
                  
        static member PointOfIntersection (a:Line2, b:Line2) =
            let ky = a.ComputeKY () - b.ComputeKY ()
            let dx = a.ComputeDX () - b.ComputeDX ()
            let x = -ky / dx
            let y = a.ComputeKY () + (x * a.ComputeDX ())
            new Vector2 (x,y)                
       
        static member Angle (a:Line2, b:Line2) = 
            let dvA = a.V1 - a.V2 
            let dvB = b.V1 - b.V2
            dvA.Normalize ()
            dvB.Normalize ()
            let dotProduct = Vector2.Dot ( dvA , dvB)
            let dot = min 1.0f (max -1.0f dotProduct)
            1.0f * acos (dot)
                                                                               
        static member Intersect (a:Line2, b:Line2) =
            let vi = Line2.PointOfIntersection (a,b)
            if a.IsBounded (vi) then
                let l1 = Line2 (vi, b.V1)
                let l2 = Line2 (vi, b.V2)
                let r = Line2.Angle (l1,l2)                                                       
                r > MathHelper.ToRadians 90.0f                    
            else
                false  
end 