﻿open System
open System.Windows
open System.Windows.Controls
open System.Windows.Media
open System.Windows.Input
open Layout

type Code = string
type Name = string
type Price = Decimal
type Product = Product of Code * Name * Price   

let products = [
        Product("8717644012208","Lynx Africa",2.49M)
        Product("5010123730215","Listerine",2.49M)
        Product("5000347009242","AquaFresh",1.99M)
        Product("5045094763863","Ibuprofen", 1.99M)]     

type Quantity = Decimal
type SaleLineItem = SaleLineItem of Product * Quantity

type Sale () =
    let mutable items = []    
    let LineTotal = function 
        SaleLineItem(Product(_,_,price),quantity) -> price * quantity
    let Total () = items |> List.sumBy LineTotal    
    member this.AddItem (item:SaleLineItem) = items <- item::items
    member this.TotalAmount = Total()       
   
type NumberEventArgs (c) = 
    inherit EventArgs()
    member this.Number = c

type NumberPad () = 
    let keys = 
        ['7';'8';'9';
         '4';'5';'6';
         '1';'2';'3';
         '-';'0';'.']
    let layout = TableLayout("*,*,*","*,*,*,*", Height=200.0)   
    let event = new Event<NumberEventArgs>()
    do  let brush = new LinearGradientBrush(Colors.Red, Colors.Pink, 90.0)
        keys
        |> Seq.map(fun c -> 
            let button = Button(Content=c.ToString(),Width=40.0,Height=40.0)            
            button.Background <- brush           
            button.Click.Add(fun e -> NumberEventArgs(c) |> event.Trigger)            
            button) 
        |> Seq.iter layout.AddItem               
    member this.Control = layout    
    member this.Pressed = event.Publish

type EntryEventArgs (s) =
    inherit EventArgs ()
    member this.Value = s

type KeyEntry () = 
    let numberPad = new NumberPad()
    let textBox = new TextBox() 
    do  textBox.FontSize <- 32.0
    let button =
        let radial = new RadialGradientBrush(Colors.LightBlue, Colors.White) 
        new Button(Content="Enter",IsDefault=true,Background=radial, Height=40.0)    
    let layout = 
        TableLayout("","200,40,40") + numberPad.Control + textBox + button
    do  numberPad.Pressed
        |> Event.add (fun x -> textBox.Text <- textBox.Text + x.Number.ToString())
    let event = new Event<_>()
    do  button.Click |> Event.add (fun e -> 
            EntryEventArgs(textBox.Text) |> event.Trigger
            textBox.Text <- "")
    member this.Entry = event.Publish
    member this.Control = layout
    member this.TextInput(text) = textBox.Text <- textBox.Text + text

[<STAThread>]
do  let sale = new Sale()
    let keyEntry = KeyEntry()
    let cyan = LinearGradientBrush(Colors.Cyan, Colors.LightCyan, 90.0)
    let blue = LinearGradientBrush(Colors.LightBlue, Colors.White, 90.0)
    let amount = TextBlock(FontSize=24.0,TextAlignment=TextAlignment.Right)
    let UpdateTotal () = amount.Text <- sale.TotalAmount.ToString("C")    
    UpdateTotal()
    let total = new GroupBox(Header="Total Amount",Content=amount,FontSize=16.0,Height=64.0)
    let panel = TableLayout("","320,*", Background=blue)+ keyEntry.Control + total
    let view = ListBox(Background=cyan,FontSize=32.0,Foreground=SolidColorBrush(Colors.DarkBlue))  
    let AddSaleItem text =
        let product = products |> Seq.tryFind (function 
            | Product(code,_,_) when code = text -> true | _ -> false)
        match product with
        | Some(Product(_,name,price)) -> 
            sale.AddItem(SaleLineItem(product.Value,1.0M))            
            view.Items.Add(String.Format("1 x {0} @ {1:C}",name,price)) |> ignore
            UpdateTotal ()
        | None -> ()
    keyEntry.Entry |> Event.add (fun e -> AddSaleItem e.Value)                       
    let layout = TableLayout("*,128","*") + view + panel
    let window = new Window(Title="Checkout",WindowState=WindowState.Maximized,WindowStyle=WindowStyle.None)
    window.TextInput |> Event.add (fun e -> e.Text |> keyEntry.TextInput)              
    window.Content <- layout            
    (new Application()).Run(window) |> ignore