﻿namespace Layout

open System
open System.Windows
open System.Windows.Controls

// Table layout panel type extends Grid for easier use in code
type TableLayout (columnDefinitions:string seq,rowDefinitions:string seq) =
    inherit Grid ()   
    /// Converts grid length string to GridLength instance
    let ParseGridLength (s:string) =
        match s.Length, s.EndsWith("*") with
        | 0, _ -> GridLength()
        | n, false -> GridLength(Double.Parse(s))
        | 1, true -> GridLength(1.0, GridUnitType.Star)
        | n, true -> GridLength(Double.Parse(s.Substring(0, n-1)), GridUnitType.Star)                  
    let ToCol len = ColumnDefinition(Width=len)    
    let ToRow len = RowDefinition(Height=len)
    do  columnDefinitions |> Seq.iter (
            ParseGridLength >> ToCol >> base.ColumnDefinitions.Add)    
    do  rowDefinitions |> Seq.iter (
            ParseGridLength >> ToRow >> base.RowDefinitions.Add)
    new (cols:string,rows:string) = 
        new TableLayout(cols.Split([|','|]),rows.Split([|','|]))
    /// Adds item to layout, automatically setting grid column and row values                   
    member this.AddItem (item:UIElement) =
        let span = match this.ColumnDefinitions.Count with | 0 -> 1 | n -> n
        let index = this.Children.Count
        item |> this.Children.Add |> ignore        
        Grid.SetColumn(item, index % span)
        Grid.SetRow(item, index / span)
    static member ( + ) (this:TableLayout, item:UIElement) = 
        this.AddItem item; this
        
(*
type TenderAmount = Decimal
type Tender =
    | Cash of TenderAmount
    | Card of TenderAmount * String * DateTime    
    static member Amount = function        
        | Cash amount -> amount
        | Card (amount,_,_) -> amount
        
type Sale () =
    let mutable items = []    
    let LineTotal = function 
        SaleLineItem(Product(_,_,price),quantity) -> price * quantity
    let Total () = items |> List.sumBy LineTotal
    let mutable tenders = []
    let Tendered () = tenders |> List.sumBy Tender.Amount
    member this.AddItem (item:SaleLineItem) = items <- item::items
    member this.TotalAmount = Total()       
    member this.AddTender (tender:Tender) = tenders <- tender::tenders                
    member this.OutstandingAmount = Total() - Tendered()         
*)        